/*+ an INKEY() function replacement with an associated screen saver
    that operates after a preset time
    Two simple pattern generator functions are supplied.  Scrn_Saver() uses
    the random number generator in Clipper Tools to produce random colors and
    screen positions.  Saver2() just produces flat color changes on the screen
    Routines issued by the author as "Freeware" for general use but the Author 
    retains coyright March 1995, Gordon A Morris
    
    To impliment the screen savers with the standard Clipper GET system, make a copy
    of the standard GETSYS.PRG supplied with Clipper and change the inkey(0) call 
    in the GetReader() function to
    
      GetApplyKey( oGet, My_inkey( ) )  //My_Inkey replaces inkey(0)
    
    compile and link the copy into your application and the screen
    saver is available in standard GET's
    
    If anyone is interested in developing this screen saver system to provide
    more saver patterns, generated with Clipper code, or to extend its use
    to MENU TO and ACHOICE replacements please e-mail the author at Compuserve
    ID 100275,1443 and they can be posted to the Clipper forum for general use
    
    Thanks, and happy screen saving !!
    
    Gordon A Morris March 1995
    */

************************************************
FUNCTION My_InKey(d_Tm)
************************************************
/*+ d_Tm is the delay time to wait before activating the screen saver
    in seconds defaults to 300 sec if not given*/

local nKey  //inkey() return value
local s_Tm  //time process started
local e_Tm  //elapsed time of the process

//set a default of 5 mins if no delay time set
d_Tm:=iif(valtype(d_Tm)=="U",300,d_Tm)

//just incase My_Inkey is called with a value of d_Tm==0
d_Tm:=iif(d_Tm==0,300,d_Tm)

nKey:=0
s_Tm:=seconds()
do while nKey==0
  nKey:=inkey()
  e_Tm:=seconds()-s_Tm
  if e_Tm > d_Tm
    Saver2()
    //Scrn_Saver()    //call the screen painting function
    s_Tm:=seconds() //re-set the start time for the process
  endif
enddo
RETURN nKey

************************************************
STATIC FUNCTION Scrn_Saver()
************************************************
/*+ a function to place random dots on the screen in random colors
    and retore the original screen when any key is pressed*/

local W_Cur         //current window handle-if using Clipper Tools Windows
local old_Scrn      //screen to save/restore
local Rw,Cl,Mr,Mc   //paint ordinates, screen max row and col values
local rC,cList      //random color number, array of color strings
local nKey

nKey:=0

//define a color string array
cList:={"w/n","w/b","w/g","w/bg","w/r","w/rb","w/gr","w/w","w/n+",;
           "w/b+","w/g+","w/bg+","w/r+","w/rb+","w/gr+","w/w+"}

//save current window - for use with Clipper Tools Window Functions
W_Cur:=Wselect()
Wselect(0)

Mr:=maxrow()
Mc:=maxcol()

//save current screen
old_Scrn:=savescreen(0,0,Mr,Mc)
do while nKey==0
  nKey:=inkey()
  Rw:=round(Mr*rand(),0)    //generate a random row position
  Cl:=round(Mc*rand(),0)    //generate a random column position
  rC:=round(15*rand()+1,0)  //select a color number
  @ Rw,Cl say " " color cList[rC]
enddo

//restore the original screen
restscreen(0,0,Mr,Mc,old_Scrn)

//restore active window - for use with Clipper Tools Window Functions
Wselect(W_Cur)
RETURN nil

************************************************
STATIC FUNCTION Saver2()
************************************************
/*+ a function to place rows and columns on the screen in colored dots
    and retore the original screen when any key is pressed
    This routine does not use Clipper Tools Functions unless the Window
    functions are used*/

local W_Cur         //current window handle-if using Clipper Tools Windows
local old_Scrn      //screen to save/restore
local Rw,Cl,Mr,Mc   //paint ordinates, screen max row and col values
local sC,cList      //color number, array of color strings
local nKey
local X,Y,Te,Ts

nKey:=0

//define a color string array
cList:={"w/n","w/b","w/g","w/bg","w/r","w/rb","w/gr","w/w","w/n+",;
           "w/b+","w/g+","w/bg+","w/r+","w/rb+","w/gr+","w/w+"}

//save current window - for use with Clipper Tools Window Functions
W_Cur:=Wselect()
Wselect(0)

Mr:=maxrow()
Mc:=maxcol()

//save current screen
old_Scrn:=savescreen(0,0,Mr,Mc)

Ts:=seconds()
Te:=0
do while nKey==0
  for Y:=0 to Mr
    for X:=0 to Mc
      nKey:=inkey()
      if nKey != 0
        exit
      endif
      Ts:=iif(Te>60,seconds(),Ts)
      Te:=seconds()-Ts
      sC:=iif(Te<5,1,int(Te/4))
      @ Y,X say " " color cList[sC]
    next
    if nKey != 0
      exit
    endif
  next
enddo

//restore the original screen
restscreen(0,0,Mr,Mc,old_Scrn)

//restore active window - for use with Clipper Tools Window Functions
Wselect(W_Cur)
RETURN nil


